package com.qmth.paper.library.common.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.paper.library.common.base.BaseEntity;
import com.qmth.paper.library.common.bean.dto.TaskPrintClassDto;
import com.qmth.paper.library.common.bean.dto.excel.BasicStudentImportDto;
import com.qmth.paper.library.common.bean.dto.excel.DescribeImportDto;
import com.qmth.paper.library.common.bean.params.BasicStudentExtrasParam;
import com.qmth.paper.library.common.bean.params.BasicStudentParams;
import com.qmth.paper.library.common.bean.result.BasicStudentResult;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.BasicStudent;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.enums.TeachBasicEnum;
import com.qmth.paper.library.common.mapper.BasicStudentMapper;
import com.qmth.paper.library.common.service.BasicStudentService;
import com.qmth.paper.library.common.util.ServletUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 学校id 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-08-02
 */
@Service
public class BasicStudentServiceImpl extends ServiceImpl<BasicStudentMapper, BasicStudent> implements BasicStudentService {
    @Resource
    private BasicStudentMapper basicStudentMapper;


    @Override
    public IPage<BasicStudentResult> basicStudentPage(String queryParams, Long collegeId, Long majorId, Set<Long> clazzIdList, int pageNumber, int pageSize) {
        Long schoolId = SystemConstant.convertIdToLong(ServletUtil.getRequestHeaderSchoolId().toString());
        queryParams = SystemConstant.translateSpecificSign(queryParams);
        return basicStudentMapper.findBasicStudentPage(new Page<>(pageNumber, pageSize), queryParams, schoolId, collegeId, majorId, clazzIdList, null);
    }

    @Override
    public BasicStudentResult findById(Long id) {
        BasicStudentResult basicStudentResult = null;
        if (SystemConstant.longNotNull(id)) {
            basicStudentResult = this.baseMapper.findById(id);
        }
        return basicStudentResult;
    }

    @Override
    public List<BasicStudentResult> basicStudentList(Long schoolId, Long collegeId, Long majorId, Long clazzId, List<String> studentCodeList) {
        return this.baseMapper.findBasicStudentList(schoolId, collegeId, majorId, clazzId, studentCodeList);
    }

    @Override
    public IPage<BasicStudent> listPage(Page<BasicStudent> page, String classId) {
        List<String> classIds = Arrays.asList(classId.split(","));
        QueryWrapper<BasicStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().in(BasicStudent::getClazzId, classIds)
                .orderByAsc(BasicStudent::getClazzId)
                .orderByAsc(BasicStudent::getStudentCode);
        return this.baseMapper.selectPage(page, queryWrapper);
    }

    @Override
    public List<TaskPrintClassDto> listByClass(Long schoolId, Set<String> stringList) {
        return this.baseMapper.listByClass(schoolId, stringList).stream()
                .peek(e -> e.setTeachBasicEnum(TeachBasicEnum.BASIC_CLAZZ))
                .collect(Collectors.toList());
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Long saveBasicStudent(BasicStudentParams basicStudentParams, SysUser requestUser) {
        List<BasicStudentParams> basicStudentParamsList = new ArrayList<>();
        basicStudentParamsList.add(basicStudentParams);
        BasicStudent basicStudent = this.editEntityHelp(basicStudentParamsList, requestUser).get(0);
        this.saveOrUpdate(basicStudent);
        return basicStudent.getId();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveOrUpdateBasicStudentBatch(List<BasicStudentParams> basicStudentParamsList, SysUser requestUser) {
        this.saveOrUpdateBatch(this.editEntityHelp(basicStudentParamsList, requestUser));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void saveBasicStudentWithExtrasList(List<BasicStudentExtrasParam> basicStudentExtrasParamList, SysUser requestUser) throws IllegalAccessException {
        Long schoolId = requestUser.getSchoolId();
        final String link = "-";
        // key = collegeName
        Map<String, Long> collegeMap = new HashMap<>();
        // key = schoolId + "-" + majorName
        Map<String, Long> majorMap = new HashMap<>();
        // key = schoolId + "-" + clazzName
        Map<String, Long> clazzMap = new HashMap<>();

        List<BasicStudentParams> addBasicStudentParamsList = new ArrayList<>();
        for (BasicStudentExtrasParam basicStudentExtrasParam : basicStudentExtrasParamList) {
            // 取值
            String studentName = basicStudentExtrasParam.getStudentName();
            String studentCode = basicStudentExtrasParam.getStudentCode();
            String phoneNumber = basicStudentExtrasParam.getPhoneNumber();
            String collegeName = basicStudentExtrasParam.getCollegeName();
            String majorName = basicStudentExtrasParam.getMajorName();
            String clazzName = basicStudentExtrasParam.getClazzName();

            // 学生信息组装
            BasicStudentParams basicStudentParams = new BasicStudentParams();
            basicStudentParams.setStudentName(studentName);
            basicStudentParams.setStudentCode(studentCode);
            basicStudentParams.setPhoneNumber(phoneNumber);

            BasicStudent tmp = this.getOne(new QueryWrapper<BasicStudent>().lambda()
                    .eq(BasicStudent::getSchoolId, requestUser.getSchoolId())
                    .eq(BasicStudent::getEnable, true)
                    .eq(BasicStudent::getStudentCode, studentCode));
            if (Objects.nonNull(tmp)) {
                basicStudentParams.setId(tmp.getId());
            }
            addBasicStudentParamsList.add(basicStudentParams);
        }
        List<BasicStudent> basicStudentList = this.editEntityHelp(addBasicStudentParamsList, requestUser);
        this.saveOrUpdateBatch(basicStudentList);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean removeBasicStudentBatch(List<Long> idList) {

        if (idList.size() == 0) {
            throw ExceptionResultEnum.ERROR.exception("请选择要删除的目标");
        }
        // 删除基础学生
        this.remove(new QueryWrapper<BasicStudent>().lambda().in(BasicStudent::getId, idList));
        // 同步删除教学学生
        this.baseMapper.deleteTeachStudentByBasicStudentIdSet(new HashSet<>(idList));
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean removeBasicStudentByCondition(String queryParams, Long collegeId, Long majorId, Set<Long> clazzIdList) {
        LambdaQueryWrapper<BasicStudent> basicStudentQueryWrapper = new QueryWrapper<BasicStudent>().lambda();
        if (SystemConstant.strNotNull(queryParams)) {
            basicStudentQueryWrapper.and(e -> e.like(BasicStudent::getStudentName, queryParams).or().like(BasicStudent::getStudentCode, queryParams));
        }
        if (SystemConstant.longNotNull(collegeId)) {
            basicStudentQueryWrapper.eq(BasicStudent::getBelongOrgId, collegeId);
        }
        if (SystemConstant.longNotNull(majorId)) {
            basicStudentQueryWrapper.eq(BasicStudent::getMajorId, majorId);
        }
        if (clazzIdList != null && clazzIdList.size() > 0) {
            basicStudentQueryWrapper.in(BasicStudent::getClazzId, clazzIdList);
        }
        List<BasicStudent> willDeleteStudent = this.list(basicStudentQueryWrapper);

        // 删除基础学生
        if (willDeleteStudent.size() > 0) {
            Set<Long> idSet = willDeleteStudent.stream().map(BaseEntity::getId).collect(Collectors.toSet());
            this.removeByIds(idSet);
            // 同步删除教学学生
            this.baseMapper.deleteTeachStudentByBasicStudentIdSet(idSet);
        }
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Map<String, Object> executeBasicStudentImportLogic(List<LinkedMultiValueMap<Integer, Object>> finalList, Map<String, Object> map) throws IllegalAccessException {
        SysUser sysUser = (SysUser) map.get(SystemConstant.SYS_USER);
        for (int i = 0; i < finalList.size(); i++) {
            LinkedMultiValueMap<Integer, Object> excelMap = finalList.get(i);
            List<Object> basicStudentImportDtoList = excelMap.get(i);

            assert basicStudentImportDtoList != null;
            if (basicStudentImportDtoList.get(0) instanceof DescribeImportDto) {
                continue;
            }
            List<BasicStudentImportDto> datasource = basicStudentImportDtoList.stream().map(e -> {
                BasicStudentImportDto basicStudentImportDto = new BasicStudentImportDto();
                BeanUtils.copyProperties(e, basicStudentImportDto);
                return basicStudentImportDto;
            }).collect(Collectors.toList());

            List<BasicStudentExtrasParam> basicStudentExtrasParamList = datasource.stream().flatMap(e -> {
                BasicStudentExtrasParam basicStudentExtrasParam = new BasicStudentExtrasParam();
                basicStudentExtrasParam.setStudentName(e.getStudentName());
                basicStudentExtrasParam.setStudentCode(e.getStudentCode());
                basicStudentExtrasParam.setPhoneNumber(e.getPhoneNumber());
                basicStudentExtrasParam.setCollegeName(e.getCollegeName());
                basicStudentExtrasParam.setMajorName(e.getMajorName());
                basicStudentExtrasParam.setClazzName(e.getClazzName());
                return Stream.of(basicStudentExtrasParam);
            }).collect(Collectors.toList());
            this.saveBasicStudentWithExtrasList(basicStudentExtrasParamList, sysUser);
            map.put("dataCount", datasource.size());
        }
        return map;
    }

    /**
     * 学生保存助手方法（应对学生保存和导入批量保存）
     *
     * @param basicStudentParamsList 学生参数
     * @param sysUser                操作人
     * @return 结果
     */
    private List<BasicStudent> editEntityHelp(List<BasicStudentParams> basicStudentParamsList, SysUser sysUser) {
        Long schoolId = sysUser.getSchoolId();
        List<BasicStudent> editEntityList = new ArrayList<>();
        for (BasicStudentParams basicStudentParams : basicStudentParamsList) {
            // 普通参数
            String studentName = basicStudentParams.getStudentName();
            String studentCode = basicStudentParams.getStudentCode();
            String phoneNumber = basicStudentParams.getPhoneNumber();
            Long clazzId = basicStudentParams.getClazzId();
            Long majorId = basicStudentParams.getMajorId();
            Long collegeId = basicStudentParams.getCollegeId();

            // 班级验证
            Long id = basicStudentParams.getId();
            // 检验学号在学校的唯一性
            BasicStudent checkCode = this.getOne(new QueryWrapper<BasicStudent>().lambda()
                    .eq(BasicStudent::getSchoolId, schoolId)
                    .eq(BasicStudent::getStudentCode, studentCode)
                    .eq(BasicStudent::getEnable, true));
            if (Objects.nonNull(checkCode)) {
                if (!checkCode.getId().equals(id)) {
                    throw ExceptionResultEnum.ERROR.exception("学生编号【" + studentCode + "】重复");
                }
            }

            // 检验有值的电话在学校的唯一性
            if (SystemConstant.strNotNull(phoneNumber)) {
                BasicStudent checkPhone = this.getOne(new QueryWrapper<BasicStudent>().lambda()
                        .eq(BasicStudent::getSchoolId, schoolId)
                        .eq(BasicStudent::getPhoneNumber, phoneNumber)
                        .eq(BasicStudent::getEnable, true));
                if (Objects.nonNull(checkPhone)) {
                    if (!checkPhone.getId().equals(id)) {
                        throw ExceptionResultEnum.ERROR.exception("学生电话【" + phoneNumber + "】重复");
                    }
                }
            }

            // 组装基础参数
            Long orgId;
            BasicStudent basicStudent = new BasicStudent();
            basicStudent.setStudentCode(studentCode);
            basicStudent.setStudentName(studentName);
            basicStudent.setPhoneNumber(phoneNumber);
            basicStudent.setBelongOrgId(collegeId);
            basicStudent.setMajorId(majorId);
            basicStudent.setClazzId(clazzId);
            basicStudent.setSchoolId(schoolId);

            // 保存
            if (!SystemConstant.longNotNull(id)) {
                // id为空 -> 新增
                // 获取分布式id
                orgId = sysUser.getOrgId();
                id = SystemConstant.getDbUuid();
                basicStudent.setId(id);

                basicStudent.setCreateId(sysUser.getId());
            } else {
                // id不为空 -> 更新
                BasicStudent old = this.getById(id);
                if (Objects.isNull(old) || old.getEnable().equals(false)) {
                    throw ExceptionResultEnum.ERROR.exception("学生主键不存在或被禁用");
                }
                orgId = old.getOrgId();
                basicStudent.setId(id);
                basicStudent.setUpdateId(sysUser.getId());
            }
            basicStudent.setOrgId(orgId);
            editEntityList.add(basicStudent);
        }
        return editEntityList;
    }
}
